jQuery.noConflict();
(function($, PLUGIN_ID) {
    'use strict';

    // plug-in config
    var config = kintone.plugin.app.getConfig(PLUGIN_ID);
    // activation
    if (config && config.activation !== 'active') {
        return;
    }

    // utilities
    var util = window.util;
    var ui = util.ui;
    var constant = util.constant;
    var TIMEZONES = constant.TIMEZONES;
    var MAX_READ_COUNT = constant.MAX_READ_COUNT;

    // params from user specifications
    var USER = kintone.getLoginUser();
    var LANG = USER.language;
    var TZ = USER.timezone;

    // container element for FullCalendar
    var CALENDAR_SELECTOR = constant.FULL_CALENDAR.SELECTOR;
    // color settings for FullCalendar
    var DEFAULT_EVENT_COLOR = constant.FULL_CALENDAR.DEFAULT_EVENT_COLOR;
    var DEFAULT_EVENT_TEXT_COLOR = constant.FULL_CALENDAR.DEFAULT_EVENT_TEXT_COLOR;

    // params for setting fields
    var alldayCheckLabels = {
        ja: config.alldayCheckLabelJa,
        en: config.alldayCheckLabelEn,
        zh: config.alldayCheckLabelZh
    };
    var START_DATETIME_FIELD = config.startDatetimeField;
    var END_DATETIME_FIELD = config.endDatetimeField;
    var TITLE_FIELD = config.titleField;
    var ALLDAY_FIELD = config.alldayCheckField;
    var ALLDAY_VALUE = util.escapeHtml(alldayCheckLabels[LANG]);
    var COLOR_FIELD = config.bgcolorField;

    // multi-language settings
    var terms = {
        en: {
            // options for FullCalendar
            monthNames: ['January', 'February', 'March', 'April', 'May', 'June',
                'July', 'August', 'September', 'October', 'November', 'December'
            ],
            monthNamesShort: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
            dayNames: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
            dayNamesShort: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
            titleFormatMonth: 'MMMM YYYY',
            titleFormatWeek: 'MMM D YYYY',
            columnFormatMonth: 'ddd',
            columnFormatWeek: 'ddd M/D',
            listDayAltFormat: 'MMM D YYYY',
            // texts for button elements
            buttonTextToday: 'Today',
            buttonTextMonth: 'Month',
            buttonTextAgendaWeek: 'Week',
            buttonTextListWeek: 'List week',
            // texts for titles and messages
            textTitle: 'Schedule title',
            textTimezone: 'Timezone',
            // texts for messages
            msgEventGetErr: 'Failed to get records',
            msgRecordUpsertErr: 'Failed to save record',
            msgRequiredFieldsErr: 'There are empty required fields',
            msgRequiredFieldErr: 'Required.',
            msgInvalidInputErr: 'Invalid input values.',
            msgInvalidFieldsInputErr: 'It needs start datetime is earlier than end datetime.'
        },
        ja: {
            // options for FullCalendar
            monthNames: ['1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月'],
            monthNamesShort: ['1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月'],
            dayNames: ['日曜日', '月曜日', '火曜日', '水曜日', '木曜日', '金曜日', '土曜日'],
            dayNamesShort: ['日', '月', '火', '水', '木', '金', '土'],
            titleFormatMonth: 'YYYY年 M月',
            titleFormatWeek: 'YYYY年 M月 D日',
            columnFormatMonth: 'ddd',
            columnFormatWeek: 'D （ddd）',
            listDayAltFormat: 'YYYY年 M月 D日',
            // texts for button elements
            buttonTextToday: '今日',
            buttonTextMonth: '月',
            buttonTextAgendaWeek: '週',
            buttonTextListWeek: '週（リスト）',
            // texts for titles and messages
            textTitle: 'スケジュールタイトル',
            textTimezone: 'タイムゾーン',
            // texts for messages
            msgEventGetErr: 'イベントレコード取得失敗',
            msgRecordUpsertErr: 'レコード保存失敗',
            msgRequiredFieldsErr: '未入力の必須フィールドがあります。',
            msgRequiredFieldErr: '必須です。',
            msgInvalidInputErr: '入力値が不正です。',
            msgInvalidFieldsInputErr: '開始日時は終了日時よりも早い必要があります。'
        }
    };
    var i18n = (LANG in terms) ? terms[LANG] : terms.en;

    // append select element of timezone to index view
    var appendTZSelect = function() {
        ui.appendToIndexHeader(
            $('<span>').text(i18n.textTimezone + ' : '),
            ui.createStyledSelect($('<select>').prop({
                id: 'tz-select'
            }))
        );

        var templateString = '<option value="{{html:#data}}" {{html:~isSelected(#data)}}>{{html:#data}}</option>';
        var template = $.templates(templateString).render(TIMEZONES, {
            isSelected: function(tz) {
                return tz === TZ ? 'selected' : '';
            }
        });
        $('#tz-select').html(template);
    };

    // get (selected) timezone value
    var getTZValue = function() {
        return $('#tz-select option:selected').val() || TZ;
    };

    // get events from current index view
    var getEvents = function(start, end, callback, errback, ofs, evts) {
        var mStart = moment.tz(start, getTZValue());
        var mEnd = moment.tz(end, getTZValue());
        var condition = kintone.app.getQueryCondition();
        if (condition) {
            condition += ' and ';
        }
        var limit = MAX_READ_COUNT;
        var offset = ofs ? ofs : 0;
        var query = condition + '(' + START_DATETIME_FIELD + ' >= "' + mStart.format() + '" and ' +
            END_DATETIME_FIELD + ' < "' + mEnd.format() + '"' + ')' + ' limit ' + limit + ' offset ' + offset;
        kintone.api(kintone.api.url('/k/v1/records', true), 'GET', {
            app: kintone.app.getId(),
            query: query
        }, function(r) {
            var recordPath = '/k/' + kintone.app.getId() + '/show#record=';
            var events = [];
            if (evts) {
                events = events.concat(evts);
            }
            events = events.concat(
                r.records.map(function(record) {
                    var allDay = false;
                    if (record[ALLDAY_FIELD].value.length > 0) {
                        allDay = true;
                    }
                    var color = record[COLOR_FIELD] && record[COLOR_FIELD].value ?
                        record[COLOR_FIELD].value : DEFAULT_EVENT_COLOR;
                    return {
                        id: record.$id.value,
                        title: record[TITLE_FIELD].value,
                        start: moment(record[START_DATETIME_FIELD].value).tz(getTZValue()).format(),
                        end: moment(record[END_DATETIME_FIELD].value).tz(getTZValue()).format(),
                        url: recordPath + record.$id.value,
                        allDay: allDay,
                        textColor: DEFAULT_EVENT_TEXT_COLOR,
                        color: color
                    };
                })
            );
            if (r.records.length < MAX_READ_COUNT) {
                callback(events);
            } else {
                getEvents(start, end, callback, errback, limit + offset, events);
            }
        }, function(e) {
            return errback(e);
        });
    };

    // clocking
    var clock = function() {
        moment.locale(LANG);
        var m = moment().tz(getTZValue());
        setTimeout(clock, 1000);
        $('#clock').text(m.format('llll'));
    };

    // append element for clocking
    var appendClock = function() {
        ui.appendToIndexHeader(
            $('<span>').prop({
                id: 'clock'
            })
        );
        clock();
    };

    // UpSert record operation in FullCalendar view
    var upsertEventRecord = function(id, start, end, allDay, title) {
        var mStart = moment.tz(start, getTZValue());
        var mEnd;
        if (!end) {
            mEnd = allDay ? mStart.clone().add(1, 'days') : mStart.clone().add(2, 'hours');
        } else {
            mEnd = moment.tz(end, getTZValue());
        }
        var method;

        // params for UpSert request
        var params = {
            app: kintone.app.getId(),
            record: {}
        };
        // set start and end datetime
        params.record[START_DATETIME_FIELD] = {};
        params.record[START_DATETIME_FIELD].value = mStart.format();
        params.record[END_DATETIME_FIELD] = {};
        params.record[END_DATETIME_FIELD].value = mEnd.format();
        // set allDay condition
        params.record[ALLDAY_FIELD] = {};
        params.record[ALLDAY_FIELD].value = [];
        if (allDay) {
            params.record[ALLDAY_FIELD].value = [ALLDAY_VALUE];
        }
        // POST or PUT
        if (!id) {
            method = 'POST';
            if (title) {
                params.record[TITLE_FIELD] = {};
                params.record[TITLE_FIELD].value = util.escapeHtml(title);
            }
        } else {
            method = 'PUT';
            params.id = id;
        }
        // UpSert request
        return kintone.api(kintone.api.url('/k/v1/record', true), method, params).then(function(r) {
            if (method === 'POST') {
                $(CALENDAR_SELECTOR).fullCalendar('refetchEvents');
            }
        }).catch(function(e) {
            swal({
                title: i18n.msgRecordUpsertErr,
                text: '',
                type: 'error'
            }, function() {
                return;
            });
        });
    };

    // default status for FullCalendar
    // https://fullcalendar.io/docs/
    var defaultStatus = {
        theme: true,
        header: {
            left: 'prev,next today',
            center: 'title',
            right: 'month,agendaWeek,listWeek'
        },
        views: {
            month: {
                buttonText: i18n.buttonTextMonth,
                columnFormat: i18n.columnFormatMonth,
                titleFormat: i18n.titleFormatMonth
            },
            agendaWeek: {
                buttonText: i18n.buttonTextAgendaWeek,
                columnFormat: i18n.columnFormatWeek,
                titleFormat: i18n.titleFormatWeek
            },
            listWeek: {
                buttonText: i18n.buttonTextListWeek,
                columnFormat: i18n.columnFormatWeek,
                titleFormat: i18n.titleFormatWeek
            },
            day: {
                columnFormat: i18n.columnFormatWeek,
                titleFormat: i18n.titleFormatWeek
            }
        },
        timeFormat: 'H:mm',
        listDayAltFormat: i18n.listDayAltFormat,

        buttonText: {
            today: i18n.buttonTextToday
        },

        monthNames: i18n.monthNames,
        monthNamesShort: i18n.monthNamesShort,
        dayNames: i18n.dayNames,
        dayNamesShort: i18n.dayNamesShort,

        defaultView: 'agendaWeek',
        defaultDate: moment().format('YYYY-MM-DD'),
        navLinks: true,
        editable: true,
        selectable: true,
        eventLimit: true,
        timezone: getTZValue(),
        aspectRatio: 1.3,
        events: function(start, end, timezone, callback) {
            getEvents(start.format(), end.format(), function(events) {
                callback(events);
            }, function(e) {
                swal({
                    title: i18n.msgEventGetErr,
                    text: '',
                    type: 'error'
                }, function() {
                    return;
                });
            });
        },
        eventDrop: function(event, delta, revertFunc) {
            var start = event.start.format();
            var end = !event.end ? null : event.end.format();
            upsertEventRecord(event.id, start, end, event.allDay);
        },
        eventResize: function(event, delta, revertFunc) {
            var start = event.start.format();
            var end = !event.end ? null : event.end.format();
            upsertEventRecord(event.id, start, end, event.allDay);
        },
        select: function(start, end, jsEvent, view, resource) {
            swal({
                title: i18n.textTitle,
                type: 'input',
                showCancelButton: true,
                showLoaderOnConfirm: true
            }, function(inputValue) {
                if (inputValue === false) {
                    return false;
                }
                upsertEventRecord(
                    null,
                    start.format(),
                    (!end ? null : end.format()),
                    !start.hasTime(),
                    inputValue
                );
            });
        }
    };

    // append calendar in customized index view
    kintone.events.on(['app.record.index.show'], function(e) {
        if (!config) {
            return e;
        }
        // if displayed view has no '#calender' element, return event
        if ($(CALENDAR_SELECTOR).length === 0) {
            return e;
        }
        // append timezone select
        appendTZSelect();
        // append clock
        appendClock();
        // append FullCalendar
        $(CALENDAR_SELECTOR).fullCalendar(defaultStatus);
        // change event for timezone
        $('#tz-select').on('change', function() {
            $(CALENDAR_SELECTOR).fullCalendar('option', 'timezone', this.value || false);
        });
    });

    // append show-only timezone in show events
    kintone.events.on([
        'app.record.detail.show',
        'app.record.create.show',
        'app.record.edit.show'
    ], function(e) {
        if (!config) {
            return e;
        }
        ui.appendToDetailHeader(
            $('<span>').text(i18n.textTimezone + ' : ' + getTZValue())
        );
        return e;
    });

    // validate input parameters for calendar
    kintone.events.on([
        'app.record.create.submit',
        'app.record.edit.submit',
        'app.record.index.edit.submit'
    ], function(e) {
        if (!config) {
            return e;
        }
        var record = e.record;

        var start = record[START_DATETIME_FIELD].value;
        var end = record[END_DATETIME_FIELD].value;

        // required fields
        if (!start || !end) {
            e.error = i18n.msgRequiredFieldsErr;
            if (!start) {
                record[START_DATETIME_FIELD].error = i18n.msgRequiredFieldErr;
            }
            if (!end) {
                record[END_DATETIME_FIELD].error = i18n.msgRequiredFieldErr;
            }
        // start is earlier than end
        } else if (moment(start).format('X') > moment(end).format('X')) {
            e.error = i18n.msgInvalidInputErr;
            record[START_DATETIME_FIELD].error = i18n.msgInvalidFieldsInputErr;
            record[END_DATETIME_FIELD].error = i18n.msgInvalidFieldsInputErr;
        // start is not equal to end
        } else if (moment(start).format('X') === moment(end).format('X')) {
            e.error = i18n.msgInvalidInputErr;
            record[START_DATETIME_FIELD].error = i18n.msgInvalidFieldsInputErr;
            record[END_DATETIME_FIELD].error = i18n.msgInvalidFieldsInputErr;
        }
        return e;
    });
})(jQuery, kintone.$PLUGIN_ID);
